# encoding: utf-8
import logging
import os
from playhouse.signals import Model as SignalsModel
from slimurl import URL
from peewee import Proxy, SqliteDatabase, MySQLDatabase, PostgresqlDatabase
from playhouse.migrate import SqliteMigrator, MySQLMigrator, PostgresqlMigrator

DB = Proxy()


class Model(SignalsModel):
    class Meta:
        database = DB


from .migrator.model import Migrations
from pypi_server.db.migrator.actions import migrate_db

log = logging.getLogger('db')


def init_sqlite(url):
    global DB

    dbfile = os.path.join("/", *url.path.split("/"))
    dirname = os.path.dirname(dbfile)

    log.info('Opening sqlite database: %s', dbfile)

    if not os.path.exists(dirname):
        os.makedirs(dirname)

    DB.initialize(SqliteDatabase(dbfile))
    log.info("Database initialized as '%s'. Checking migrations...", dbfile)
    return DB, SqliteMigrator(DB)


def init_mysql(url):
    global DB

    db_name = url.path.strip("/")

    DB.initialize(MySQLDatabase(
        database=db_name,
        user=url.user or '',
        password=url.password or '',
        host=url.host,
        autocommit=bool(url.get('autocommit', True)),
        autorollback=bool(url.get('autorollback', True))
    ))
    log.info("Database initialized as '%s'. Checking migrations...", db_name)
    return DB, MySQLMigrator(DB)


def init_postgres(url):
    global DB

    db_name = url.path.strip("/")

    DB.initialize(PostgresqlDatabase(
        database=db_name,
        user=url.user or '',
        password=url.password or '',
        host=url.host,
        autocommit=bool(url.get('autocommit', True)),
        autorollback=bool(url.get('autorollback', True))
    ))
    log.info("Database initialized as '%s'. Checking migrations...", db_name)
    return DB, PostgresqlMigrator(DB)


DB_ENGINES = {
    'sqlite': init_sqlite,
    'sqlite3': init_sqlite,
    'mysql': init_mysql,
    'postgresql': init_postgres,
    'postgres': init_postgres,
    'pg': init_postgres,
}


def init_db(db_url):
    db_url = URL(db_url)
    DB, migrator = DB_ENGINES.get(
        db_url.scheme,
        lambda x: log.fatal("Unable to fund database driver")
    )(db_url)

    DB.create_tables([Migrations], safe=True)
    migrate_db(DB, Migrations, migrator)



# encoding: utf-8
from pypi_server.db.migrator import migration
from pypi_server.db.packages import Package, PackageFile, PackageVersion


@migration(3)
def create_package(migrator, db):
    db.create_tables([Package])


@migration(4)
def create_package_version(migrator, db):
    db.create_tables([PackageVersion])


@migration(5)
def create_package_file(migrator, db):
    db.create_tables([PackageFile])



# encoding: utf-8
from playhouse.migrate import migrate
from pypi_server.db.migrator import migration
from pypi_server.db.packages import PackageFile


@migration(6)
def add_url_fileld(migrator, db):
    try:
        PackageFile.select(PackageFile.url).where(PackageFile.url == None).count()
    except:
        migrate(
            migrator.add_column(
                'packagefile',
                'url',
                PackageFile.url
            )
        )


@migration(7)
def add_fetched_fileld(migrator, db):
    try:
        PackageFile.select(PackageFile.fetched).where(PackageFile.fetched == None).count()
    except:
        migrate(
            migrator.add_column(
                'packagefile',
                'fetched',
                PackageFile.fetched
            )
        )



# encoding: utf-8
from tornado.gen import coroutine
from tornado.ioloop import IOLoop
from tornado.log import app_log as log
from tornado.web import HTTPError
from pypi_server.handlers.base import BaseHandler, threaded


try:
    import ujson as json
except ImportError:
    import json


class JSONHandler(BaseHandler):
    _json = None

    @coroutine
    def prepare(self, *args, **kwargs):
        content_type = self.request.headers.get('Content-Type', '')

        if self.request.method.upper() in ('POST', 'PUT'):
            if 'application/json' not in content_type:
                raise HTTPError(400)

            self._json = yield self.thread_pool.submit(self._from_json, self.request.body)
        else:
            self._json = None

    @classmethod
    @threaded
    def _to_json(cls, data):
        return json.dumps(data)

    def _on_async_response_fail(self, result):
        if isinstance(result, Exception):
            log.exception(result)
            if not self._finished:
                self.send_error(500)

    def response(self, data):
        self.set_header('Content-Type', 'application/json')
        self._auto_finish = False
        IOLoop.current().add_future(
            self._async_response(data),
            self._on_async_response_fail
        )

    @coroutine
    def _async_response(self, data):
        if not self._finished:
            self.finish((yield self._to_json(data)))

    @staticmethod
    def _from_json(data):
        return json.loads(data)

    @property
    def json(self):
        return self._json


import pypi_server.handlers.api.login
import pypi_server.handlers.api.users
import pypi_server.handlers.api.user
import pypi_server.handlers.api.packages
import pypi_server.handlers.api.package
import pypi_server.handlers.api.version



#!/usr/bin/env python
# encoding: utf-8
import peewee as p
from playhouse.fields import PasswordField
from pypi_server.db import Model


class Users(Model):
    login = p.CharField(max_length=255, unique=True, index=True, null=False)
    password = PasswordField(iterations=10, null=False)
    disabled = p.BooleanField(default=False, null=False)
    is_admin = p.BooleanField(default=False, null=False)
    email = p.CharField(index=True)

    @classmethod
    def check(cls, login, password):
        q = cls.select(cls.id, cls.password).where(
            cls.disabled == False,
            cls.login == str(login),
        )

        user = q.limit(1)
        user = user[0] if user else None

        if user and user.password.check_password(password):
            return cls.get(id=user.id)
        else:
            raise p.DoesNotExist("User doesn't exists")



# encoding: utf-8
import datetime
import peewee
from pypi_server.db import Model


class Migrations(Model):
    name = peewee.CharField(max_length=255, null=False, index=True, unique=True)
    ts = peewee.DateTimeField(default=datetime.datetime.now, null=False)



# encoding: utf-8
from pypi_server.db.packages import Package, PackageVersion
from pypi_server.handlers import route
from pypi_server.handlers.api import JSONHandler
from pypi_server.handlers.api.login import authorization_required
from pypi_server.handlers.base import threaded
from pypi_server.handlers.pypi.proxy.client import normalize_package_name
from tornado.gen import coroutine
from tornado.web import HTTPError


@route('/api/v1/package/(?P<package>[\w\.\d\-\_]+)/?')
class PackageHandler(JSONHandler):
    @authorization_required()
    @coroutine
    def get(self, package):
        pkg = yield self.get_package(package)
        self.response((yield self.package_info(pkg)))

    @staticmethod
    @threaded
    def package_info(pkg):
        return {
            'name': pkg.lower_name,
            'owner': pkg.owner_id,
            'proxy': pkg.is_proxy,
            'versions': list(
                map(
                    str,
                    sorted(
                        map(
                            lambda x: x.version,
                            pkg.packageversion_set
                        ),
                        reverse=True
                    )
                )
            )
        }

    @staticmethod
    @threaded
    def get_package(package):
        package = normalize_package_name(package)
        q = Package.select().join(PackageVersion).where(Package.lower_name == package).limit(1)

        if not q.count():
            raise HTTPError(404)

        return q[0]

    @authorization_required(is_admin=True)
    @coroutine
    def delete(self, package):
        pkg = yield self.get_package(package)
        yield self.thread_pool.submit(pkg.delete_instance, recursive=True)
        self.response({
            'package': pkg.lower_name,
            'deleted': True,
        })

    @authorization_required(is_admin=True)
    @coroutine
    def put(self, package):
        pkg = yield self.get_package(package)
        pkg.owner = self.json.get('owner', pkg.owner_id)

        if pkg.is_dirty():
            yield self.thread_pool.submit(pkg.save)

        self.response((yield self.package_info(pkg)))



# encoding: utf-8
import re
from tornado.web import HTTPError
from pypi_server.db.users import Users
from pypi_server.handlers import route
from pypi_server.handlers.base import threaded
from pypi_server.handlers.api import JSONHandler
from pypi_server.handlers.api.login import authorization_required


LOGIN_EXP = re.compile("^[\d\w]+$")
EMAIL_EXP = re.compile("^[^\@]+\@\S+$")


@route('/api/v1/users/?')
class UsersHandler(JSONHandler):
    @authorization_required(is_admin=True)
    @threaded
    def get(self):
        self.response(
            list(
                map(
                    lambda x: dict(
                        id=x.id,
                        login=x.login,
                        email=x.email,
                        is_admin=x.is_admin,
                        disabled=x.disabled,
                    ),
                    Users.select(
                        Users.id,
                        Users.login,
                        Users.email,
                        Users.is_admin,
                        Users.disabled
                    )
                )
            )
        )

    @authorization_required(is_admin=True)
    @threaded
    def post(self):
        try:
            login = self.json["login"]
            email = self.json["email"]
            is_admin = bool(self.json.get("is_admin", 0))
            password = self.json["password"]

            assert password and len(password) > 3
            assert LOGIN_EXP.match(login)
            assert EMAIL_EXP.match(email)
        except (KeyError, AssertionError):
            raise HTTPError(400)

        if Users.select().where(Users.login == login).count():
            raise HTTPError(409)

        user = Users(
            login=login,
            email=email,
            is_admin=is_admin,
            password=password,
        )

        user.save()

        self.response({
            'id': user.id,
            'login': user.login,
            'email': user.email,
            'is_admin': user.is_admin,
        })



# encoding: utf-8
import base64
import hashlib
import logging
from functools import wraps
from peewee import DoesNotExist
from pypi_server import PY2
from pypi_server.db import DB
from pypi_server.handlers.pypi.proxy.client import PYPIClient
from tornado.gen import coroutine, Task, maybe_future, Return
from tornado.web import asynchronous, HTTPError
from tornado.httpclient import AsyncHTTPClient
from pypi_server.handlers import route
from pypi_server.handlers.base import BaseHandler, threaded
from pypi_server.http_cache import HTTPCache
from pypi_server.cache import Cache, HOUR, MONTH
from pypi_server.db.packages import Package, PackageVersion, PackageFile, HashVersion
from pypi_server.db.users import Users


if PY2:
    from urllib import unquote_plus
else:
    from urllib.parse import unquote_plus


log = logging.getLogger(__name__)


@threaded
def write_file(pkg_file, data):
    with pkg_file.open('w+') as f:
        f.write(data)

    pkg_file.fetched = True

    assert hashlib.md5(data).hexdigest() == pkg_file.md5

    pkg_file.save()
    return pkg_file


@route(r"/simple/(?P<package>\S+)/(?P<version>\S+)/(?P<filename>\S+)")
@route(r"/package/(?P<package>\S+)/(?P<version>\S+)/(?P<filename>\S+)")
class FileHandler(BaseHandler):
    CHUNK_SIZE = 2 ** 16
    HTTP_CLIENT = AsyncHTTPClient()

    @asynchronous
    @HTTPCache(MONTH, use_expires=True, expire_timeout=MONTH)
    @coroutine
    def get(self, package, version, filename):
        try:
            package = yield PYPIClient.find_real_name(package)
        except LookupError:
            package = yield self.thread_pool.submit(Package.find, package)

        try:
            pkg_file = yield self.find_file(package, version, filename)
            if not pkg_file.fetched:
                yield self.fetch_remote_file(pkg_file)
        except LookupError:
            self.send_error(404)
        else:
            self.set_header("MD5", pkg_file.md5)
            self.set_header("ETag", pkg_file.md5)
            self.set_header("Content-Length", pkg_file.size)
            self.set_header("Content-Type", 'application/octet-stream')
            self.set_header("Date", pkg_file.ts.strftime("%a, %d %b %Y %H:%M:%S %Z"))

            with pkg_file.open() as f:
                reader = threaded(f.read)

                data = yield reader(self.CHUNK_SIZE)
                self.write(data)
                yield Task(self.flush)

                while data:
                    data = yield reader(self.CHUNK_SIZE)
                    self.write(data)
                    yield Task(self.flush)

            self.finish()

    @threaded
    @Cache(HOUR, ignore_self=True)
    def find_file(self, package, version, filename):
        if not isinstance(package, Package):
            package = Package.find(package)

        pkg_file = PackageFile.select(
            PackageFile.id
        ).join(
            Package
        ).join(
            PackageVersion
        ).where(
            PackageFile.package == package,
            PackageFile.basename == filename,
            PackageVersion.version == HashVersion(version),
        ).limit(1)

        if not pkg_file.count():
            raise LookupError("Not found")

        return PackageFile.get(id=pkg_file[0].id)

    @classmethod
    @coroutine
    def fetch_remote_file(cls, pkg_file):
        response = yield cls.HTTP_CLIENT.fetch(pkg_file.url)
        yield write_file(pkg_file, response.body)


@threaded
def check_password(login, password):
    try:
        user = Users.check(login, password)
    except DoesNotExist:
        raise LookupError('User not found')

    return user


def authorization_required(func):
    @wraps(func)
    @coroutine
    def wrap(self, *args, **kwargs):
        auth_header = self.request.headers.get('Authorization')
        if not auth_header:
            self.set_header('WWW-Authenticate', 'Basic realm="pypi"')
            self.set_status(401)
            raise Return(self.finish("Authorization required"))

        auth_type, data = auth_header.split()
        if auth_type.lower() != 'basic':
            raise Return(self.send_error(400))

        username, password = map(unquote_plus, base64.b64decode(data).split(":"))
        try:
            self.current_user = yield check_password(username, password)
        except LookupError:
            raise HTTPError(403)

        result = yield maybe_future(func(self, *args, **kwargs))
        raise Return(result)

    return wrap


@route(r"/pypi/?")
class XmlRPC(BaseHandler):
    METADATA_KEYS = {
        u"name",
        u"version",
        u"stable_version",
        u"author",
        u"author_email",
        u"maintainer",
        u"maintainer_email",
        u"home_page",
        u"license",
        u"summary",
        u"description",
        u"keywords",
        u"platform",
        u"download_url",
        u"classifiers",
        u"requires",
        u"requires_dist",
        u"provides",
        u"provides_dist",
        u"requires_external",
        u"requires_python",
        u"obsoletes",
        u"obsoletes_dist",
        u"project_url",
    }

    @coroutine
    def post(self):
        if '\r' not in self.request.body:
            self.request.body = self.request.body.replace("\n", "\r\n")
            self.request._parse_body()

        try:
            action = self.get_body_argument(':action')
            self.request.body_arguments.pop(':action')

            log.debug("Request to call action: %s", action)
            method = getattr(self, "action_{0}".format(action), self._action_not_found)
        except:
            raise HTTPError(400)

        log.info("Calling action: %s", action)
        yield maybe_future(method())

    @staticmethod
    def _action_not_found():
        raise HTTPError(400)

    @authorization_required
    @threaded
    def action_submit(self):
        name = self.get_body_argument('name')

        q = Package.select().where(
            Package.name == name,
            Package.owner == self.current_user
        )

        if q.count():
            raise HTTPError(409)

        q = Package.select().where(
            Package.name == name,
            Package.owner != self.current_user
        )

        if q.count():
            raise HTTPError(403)

        pkg = Package(name=name, lower_name=name.lower(), owner=self.current_user)
        pkg.save()

    def action_verify(self):
        pass

    @authorization_required
    @threaded
    def action_file_upload(self):
        with DB.transaction():
            package_name = self.get_body_argument('name').lower()

            if not Package.select().where(Package.lower_name == package_name).count():
                raise HTTPError(404)

            package = Package.get(lower_name=package_name)
            version = package.create_version(self.get_body_argument('version'))

            uploaded_file = self.request.files['content'][0]
            if not str(hashlib.md5(uploaded_file.body).hexdigest()) == str(self.get_body_argument('md5_digest')):
                raise HTTPError(406)

            list_values = (u'classifiers', u'keywords')
            for key in self.METADATA_KEYS:
                val = self.get_body_argument(key, 'UNKNOWN')
                if key in list_values and not isinstance(val, (list, tuple)):
                    val = [val]

                setattr(version, key, val)

            version.save()

            pkg_file = version.create_file(uploaded_file.filename)
            pkg_file.fetched = True
            pkg_file.md5 = str(self.get_body_argument('md5_digest'))

            if pkg_file.exists():
                raise HTTPError(409)

            with pkg_file.open("wb+") as f:
                f.write(uploaded_file.body)

            pkg_file.save()



# encoding: utf-8
import os
import sys

PY2 = (sys.version_info < (3,))


author_info = [
    ("Dmitry Orlov", "me@mosquito.su")
]

version_info = (0, 2, 0)

__version__ = ".".join(map(str, version_info))
__author__ = ", ".join("{0} <{1}>".format(*author) for author in author_info)

ROOT = os.path.abspath(os.path.dirname(__file__))



# encoding: utf-8
from pypi_server.db.migrator.migrations import create_tables
from pypi_server.db.migrator.migrations import create_default_user
from pypi_server.db.migrator.migrations import create_package_tables
from pypi_server.db.migrator.migrations import packagefile_add_url_and_fetched
from pypi_server.db.migrator.migrations import packagefile_basename_unique



# encoding: utf-8
import logging
import signal
from time import time
from functools import wraps
from inspect import isgeneratorfunction
from collections import namedtuple, defaultdict
from multiprocessing import RLock
from tornado.gen import Return, sleep
from tornado.ioloop import IOLoop
from tornado.locks import Lock


FunctionType = type(lambda: None)
log = logging.getLogger("cache")


class Result(namedtuple('ResultBase', "ts result")):
    def __new__(cls, result):
        return super(Result, cls).__new__(cls, ts=time(), result=result)


class Cache(object):
    __slots__ = ('timeout', 'ignore_self', 'oid')

    CACHE = {}
    FUTURE_LOCKS = defaultdict(Lock)
    RLOCKS = defaultdict(RLock)

    def __init__(self, timeout, ignore_self=False, oid=None):
        self.timeout = timeout
        self.ignore_self = ignore_self
        self.oid = oid

    @staticmethod
    def hash_func(key):
        if isinstance(key, FunctionType):
            return ".".join((key.__module__, key.__name__))
        else:
            return str(key)

    @classmethod
    def invalidate(cls, func):
        fkey = cls.hash_func(func)

        hash_fkey = hash(fkey)
        for key in filter(lambda x: hash(x[0]) == hash_fkey, cls.CACHE):
            log.debug('INVALIDATING Cache for %r', key)
            cls.CACHE.pop(key, -1)
            cls.FUTURE_LOCKS.pop(key, -1)
            cls.RLOCKS.pop(key, -1)

    @classmethod
    def invaliate_all(cls, *args, **kwargs):
        log.warning("Invalidating all memory cache.")
        cls.CACHE.clear()
        cls.FUTURE_LOCKS.clear()
        cls.RLOCKS.clear()

    def __call__(self, func):
        is_generator = isgeneratorfunction(func)
        key = self.oid or self.hash_func(func)

        def get_hash(func, args, kwargs):
            return tuple(
                map(
                    hash,
                    (
                        key,
                        tuple(map(hash, args[1:] if self.ignore_self else args)),
                        tuple(map(lambda x: tuple(map(hash, x)), kwargs.items())),
                        is_generator,
                    )
                )
            )

        @wraps(func)
        def wrap(*args, **kwargs):
            io_loop = IOLoop.current()

            args_key = get_hash(func, args, kwargs)
            start_time = io_loop.time()

            with self.RLOCKS[args_key]:
                ret = self.CACHE.get(args_key)

                if isinstance(ret, Result):
                    log.debug("HIT Cache [%s] %r", key, args_key)
                    return ret.result

                ret = Result(func(*args, **kwargs))
                self.CACHE[args_key] = ret

                io_loop.add_callback(
                    io_loop.call_later,
                    self.timeout,
                    self._expire,
                    key,
                    args_key
                )

                log.debug(
                    "MISS Cache [%s] %r. Execution time %.6f sec.",
                    key,
                    args_key,
                    io_loop.time() - start_time
                )
                return ret.result

        @wraps(func)
        def wrap_gen(*args, **kwargs):
            io_loop = IOLoop.current()

            args_key = get_hash(func, args, kwargs)
            start_time = io_loop.time()

            with (yield self.FUTURE_LOCKS[args_key].acquire()):
                ret = self.CACHE.get(args_key)

                if isinstance(ret, Result):
                    yield sleep(0)
                    log.debug("HIT Cache [%s] %r", key, args_key)
                    raise Return(ret.result)

                gen = func(*args, **kwargs)

                try:
                    f = next(gen)
                    while True:
                        res = yield f
                        f = gen.send(res)
                except Return as e:
                    ret = Result(e.value)
                except StopIteration as e:
                    ret = Result(getattr(e, 'value', None))

                if ret.result:
                    self.CACHE[args_key] = ret
                    io_loop.add_callback(
                        io_loop.call_later,
                        self.timeout,
                        self._expire,
                        key,
                        args_key
                    )

                    log.debug(
                        "MISS Cache [%s] %r. Execution time %.6f sec.",
                        key,
                        args_key,
                        io_loop.time() - start_time
                    )
                else:
                    log.warning(
                        "Generator '%s' no return any value. Cache ignoring.",
                        self.hash_func(func)
                    )

                    log.debug("INVALID Cache [%s] %r", key, args_key)

                raise Return(ret.result)

        return wrap_gen if is_generator else wrap

    def _expire(self, key, args_key):
        self.CACHE.pop(args_key)
        log.debug("EXPIRED Cache [%s] %r", key, args_key)


signal.signal(signal.SIGUSR1, Cache.invaliate_all)
signal.signal(signal.SIGUSR2, Cache.invaliate_all)


MINUTE = 60
HOUR = MINUTE * 60
DAY = 24 * HOUR
WEEK = 7 * DAY
MONTH = 4 * WEEK



# encoding: utf-8
import uuid
import errno
import signal
import pwd
import os
from slimurl import URL
from tornado.ioloop import IOLoop
from tornado.httpserver import HTTPServer
from tornado.log import gen_log as log
from tornado.options import options, define
from tornado.process import cpu_count
from tornado.concurrent import futures
from tornado.web import Application
from tornado.httpclient import AsyncHTTPClient
from tornado.ioloop import PeriodicCallback
from pypi_server import ROOT
from pypi_server.cache import HOUR
from pypi_server.handlers.pypi.proxy.client import PYPIClient
from pypi_server.db import init_db
from pypi_server.db.packages import PackageFile
from pypi_server import handlers


define('config', help="Configuration file")

define("address", help="Listen address (default 127.0.0.1) [ENV:ADDRESS]",
       default=os.getenv('ADDRESS', "127.0.0.1"))

define("port", help="Listen port (default 8080) [ENV:PORT]",
       type=int, default=int(os.getenv('PORT', '8080')))

define("debug", help="Use for attach a debugger",
       default=bool(os.getenv("DEBUG")), type=bool)

define("gzip", help="Compress responses (default False) [ENV:GZIP]",
       default=bool(os.getenv("GZIP", '0')), type=bool)

define("proxy-mode", help="Process X-headers on requests (default True) [ENV:PROXY_MODE]",
       default=bool(os.getenv('PROXY_MODE', '1')), type=bool)

define("pool-size", help="Thread pool size (default cou_count * 2) [ENV:POOL_SIZE]",
       type=int, default=int(os.getenv('POOL_SIZE', cpu_count() * 2)))

define("secret", help="Cookie secret (default random) [ENV:SECRET]",
       default=os.getenv("SECRET", uuid.uuid4().bytes))

define("user", help="Change UID of current process (not change by default)", default=None)

define(
    "storage", help="Packages storage (default $CWD/packages) [ENV:STORAGE]", type=str,
    default=os.path.abspath(
        os.getenv(
            "STORAGE",
            os.path.join(os.path.abspath(os.path.curdir), 'packages')
        )
    )
)

define(
    "database", help="Application database (default sqlite:///{storage}/metadata.db) [ENV:DB]",
    type=URL,
    default=os.getenv(
        "DB",
        URL(
            "sqlite://{0}".format("/".join(
                os.path.split(os.path.join(os.path.abspath(os.path.curdir), 'packages', 'metadata.db'))
            ))
        )
    )
)

define("max_http_clients",
       help="Maximum HTTP Client instances for proxy requests (default 25) [ENV:MAX_CLIENTS]",
       default=int(os.getenv("MAX_CLIENTS", '25')), type=int)

define("pypi_server",
       help="PYPI service url. Using for proxy. (default https://pypi.python.org/) [ENV:PYPY_SERVER]",
       default=URL(os.getenv("PYPI_SERVER", 'https://pypi.python.org/')), type=URL)


def run():
    options.parse_command_line()

    if options.config:
        options.parse_config_file(options.config)

    options.storage = os.path.abspath(options.storage)

    if os.getuid() == 0 and options.user:
        pw = pwd.getpwnam(options.user)
        uid, gid = pw.pw_uid, pw.pw_gid
        log.info("Changind user to %s [%s:%s]", options.user, uid, gid)
        os.setgid(uid)
        os.setuid(uid)

    try:
        if not all(f(options.storage) for f in (os.path.exists, os.path.isdir)):
            log.info('Creating new package storage directory: "%s"', options.storage)
            os.makedirs(options.storage)

        def on_interrupt(*args):
            log.warning("Receiving interrupt signal. Application will be stopped.")
            exit(errno.EINTR)

        log.debug("Preparing signal handling")
        for sig in (signal.SIGINT, signal.SIGTERM, signal.SIGQUIT):
            signal.signal(sig, on_interrupt)

        log.debug("Creating application instance")
        app = Application(
            base_dir=ROOT,
            debug=options.debug,
            reload=options.debug,
            cookie_secret=options.secret,
            template_path=os.path.join(ROOT, 'templates'),
            default_handler_class=handlers.DefaultHandler,
            gzip=options.gzip,
            handlers=handlers.ROUTES,
            options=options
        )

        log.debug("Creating IOLoop instance.")
        io_loop = IOLoop.current()

        io_loop.run_sync(lambda: init_db(options.database))

        log.info("Init thread pool with %d threads", options.pool_size)
        handlers.base.BaseHandler.THREAD_POOL = futures.ThreadPoolExecutor(options.pool_size)

        AsyncHTTPClient.configure(None, max_clients=options.max_http_clients)

        PYPIClient.configure(
            options.pypi_server,
            handlers.base.BaseHandler.THREAD_POOL
        )

        pypi_updater = PeriodicCallback(PYPIClient.packages, 12 * HOUR * 1000, io_loop)

        io_loop.add_callback(PYPIClient.packages)
        io_loop.add_callback(pypi_updater.start)

        log.info("Starting server http://%s:%d/", options.address, options.port)
        http_server = HTTPServer(app, xheaders=options.proxy_mode)
        http_server.listen(options.port, address=options.address)

        log.debug('Setting "%s" as storage', options.storage)
        PackageFile.set_storage(options.storage)

        log.debug("Starting main loop")
        io_loop.start()
    except Exception as e:
        log.fatal("Exception on main loop:")
        log.exception(e)
        exit(1)
    else:
        exit(0)


if __name__ == '__main__':
    run()



#!/usr/bin/env python
# encoding: utf-8
from __future__ import absolute_import, print_function
import os
import pypi_server
from setuptools import setup, find_packages


def walker(base, *paths):
    file_list = set([])
    cur_dir = os.path.abspath(os.curdir)

    os.chdir(base)
    try:
        for path in paths:
            for dname, dirs, files in os.walk(path):
                for f in files:
                    file_list.add(os.path.join(dname, f))
    finally:
        os.chdir(cur_dir)

    return list(file_list)


data_files = ()

if os.geteuid() == 0:
    data_files = (
        ("/etc/systemd/system", (os.path.join('contrib', 'pypi-server.service'),)),
        ("/etc", (os.path.join('contrib', 'pypi-server.conf.example'),))
    )


setup(
    name='pypi-server',
    version=pypi_server.__version__,
    author=pypi_server.__author__,
    author_email=", ".join(map(lambda x: x[1], pypi_server.author_info)),
    url="https://github.com/mosquito/pypi-server/",
    license="MIT",
    description="Tornado PyPi server",
    long_description=open('README.rst').read(),
    platforms="all",
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Communications :: Email',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Version Control',
        'Topic :: System',
        'Topic :: System :: Software Distribution',
    ),

    include_package_data=True,
    zip_safe=False,
    package_data={
        'pypi_server': walker(os.path.dirname(pypi_server.__file__), 'static', 'templates'),
    },
    data_files=data_files,
    entry_points={
        'console_scripts': [
            'pypi-server = pypi_server.server:run',
        ],
    },
    packages=find_packages(exclude=('tests',)),
    install_requires=(
        'tornado>=4.3',
        'tornado-xmlrpc',
        'slimurl',
        'peewee',
        'bcrypt',
        'lxml',
        'futures',
    ),
    extras_require={
        'mysql': ['mysql-python'],
        'postgres': ['psycopg2'],
    }
)



# encoding: utf-8
import logging

log = logging.getLogger("db.migrator")
MIGRATIONS = []


def migration(version):
    def decorator(func):
        global MIGRATIONS

        MIGRATIONS.append((
            version,
            "{0.__module__}.{0.__name__}".format(func),
            func
        ))

        return func
    return decorator


import pypi_server.db.migrator.migrations
